/********************************************************************
 * FileName:        V_DEF_MCP19111.h
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro?Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/


//******************************************************************************
// Dynamic #define definition (modified by the GUI)
//******************************************************************************

//SF part register
#define V_T2CON           0X04
#define V_PR2             0X19
#define V_PWMRL           0X19
#define V_PWMPHL          0X00
//OV register
#define V_OVFCON          0X90
#define V_OVCCON          0XCF
#define V_OUVCON          0X43
#define V_OOVCON          0XCF
#define V_ABECON          0X0F
//input under voltage
#define V_VINLVL          0X1A

//multi phase config
#define V_BUFFCON         0X03
//Dead time
#define V_PE1             0X00
#define V_DEADCON         0X13
//Compensation
#define V_CMPZCON         0X0A
#define V_SLPCRCON        0X42
//Current sense gain
#define V_CSGSCON         0X05
//Current sense DC gain
#define V_CSDGCON         0X00
//Over Current
#define V_OCCON           0XE0
//Others
#define V_VZCCON          0X80
//periph activation
#define V_ATSTCON         0X80
//Slave Mode Err
#define V_SLVGNCON        0X00
//SOFT_START
#define V_TMR1L           0X73
#define V_TMR1H           0XBA

//----- Variable used by firmware only

#define V_OPERATION 0X00
#define V_ON_OFF_CONFIG 0X1F
#define V_VOUT_COMMAND_L 0X49
#define V_VOUT_COMMAND_H 0X03
#define V_VOUT_MODE 0X18
#define V_VOUT_TRIM_L 0XFD
#define V_VOUT_TRIM_H 0X07
#define V_VOUT_MARGIN_HIGH_L 0X5E
#define V_VOUT_MARGIN_HIGH_H 0X01
#define V_VOUT_MARGIN_LOW_L 0X13
#define V_VOUT_MARGIN_LOW_H 0X01
#define V_MAX_DUTY_L 0X64
#define V_MAX_DUTY_H 0X00
#define V_FREQUENCY_SWITCH_L 0X9A
#define V_FREQUENCY_SWITCH_H 0X08
#define V_VIN_ON_L 0X00
#define V_VIN_ON_H 0XF0
#define V_VIN_OFF_L 0X00
#define V_VIN_OFF_H 0XF0
#define V_INTERLEAVE_L 0X10
#define V_INTERLEAVE_H 0X00
#define V_VOUT_OV_FAULT_LIMIT_L 0X45
#define V_VOUT_OV_FAULT_LIMIT_H 0X03
#define V_VOUT_OV_FAULT_RESPONSE 0X78
#define V_VOUT_UV_FAULT_LIMIT_L 0X0F
#define V_VOUT_UV_FAULT_LIMIT_H 0X01
#define V_VOUT_UV_FAULT_RESPONSE 0X78
#define V_IOUT_OC_FAULT_LIMIT_L 0X40
#define V_IOUT_OC_FAULT_LIMIT_H 0XF8
#define V_IOUT_OC_FAULT_RESPONSE 0XC0
#define V_OT_WARN_LIMIT_L 0X64
#define V_OT_WARN_LIMIT_H 0X00
#define V_POWER_GOOD_ON_L 0X13
#define V_POWER_GOOD_ON_H 0X01
#define V_POWER_GOOD_OFF_L 0XFA
#define V_POWER_GOOD_OFF_H 0X00
#define V_TON_RISE_L 0XC8
#define V_TON_RISE_H 0X00
#define V_STATUS_WORD_L 0X00
#define V_STATUS_WORD_H 0X00
#define V_STATUS_VOUT 0X00
#define V_STATUS_CML 0X00
#define V_READ_VIN_L 0X3A
#define V_READ_VIN_H 0XF0
#define V_READ_VOUT_L 0X38
#define V_READ_VOUT_H 0X01
#define V_READ_IOUT_L 0X03
#define V_READ_IOUT_H 0XF8
#define V_READ_TEMPERATURE_1_L 0X02
#define V_READ_TEMPERATURE_1_H 0X00
#define V_READ_DUTY_CYCLE_L 0X02
#define V_READ_DUTY_CYCLE_H 0X00
#define V_PMBUS_REVISION 0X01
#define V_REVISION_L 0X02
#define V_REVISION_H 0X01


#define V_ON_VINLVL 0X1A
// I2C Slave address definition
#define V_SLAVE_ADDR             0X28
#define SLAVE_ADDR               V_SLAVE_ADDR
// SMBAlert address definition
#define V_SLAVE_ALERT_ADDR       0X21
#define SLAVE_ALERT_DIS_ADDR     0x00

#define SLAVE_ADDR_WRITE        SLAVE_ADDR // slave write address
#define SLAVE_ADDR_READ         SLAVE_ADDR|0X01 // slave read address

//Pin selection used to affect pin in function of V_NUM_XXX_PIN (only if < 0XFF=)
#define V_NUM_EN_PIN         0X0B
#define V_NUM_SMBALERT_PIN   0X04
#define V_NUM_I2C_ADDR_PIN   0X85

#define V_SETTING_RON        0X05
#define V_PMB_INIT           0X01
#define V_PMB_FLASH_INIT     0X00

#define V_FIRMWARE_REV       0X64 //Rev = V_FIRMWARE_REV / 100


//******************************************************************************
// PIN (IO) configuration
//******************************************************************************
#include "pin_macro.h"

//******************************************************************************
// Varius initialisatsions values
//******************************************************************************
#define NOT_OV_UV_MASK        0X3F
//TIMER 1 CONFIG
#define DEF_T1CON         0X12
//INTERRUPT CONFIG
#define DEF_INTCON        0XC8
#define  DEF_PIE1         0xBD
#define  DEF_PIE2         0xFE


//******************************************************************************
// PMBus PART
//******************************************************************************
#define TEMP_COEFF 142
#define TEMP_OFFSET 70

//Reg value to real volt conversion
#define ADC_VOLT_MULT 204

//ADC SETUP
#define DEF_ADCON0        0X11
#define DEF_ADCON1        0X50
#define DEF_SSPSTAT       0x00
#define DEF_OPTION_REG    0x28

//MSSP SETUP
#define DEF_SSPCON1       0x26
#define DEF_SSPCON2       0x00
#define DEF_SSPCON3       0x05

#define DEF_OV_UV_RECOVERY_TIME 0x3FF
#define DEF_OV_UV_DELAY_TIME 0xFF

#define DEF_MAIN_LOOP_REQ_INIT 0xFF
#define DEF_EN_PIN_RECOVERY_TIME 0xF
