/********************************************************************
 * FileName:        adc_monitor.c
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/

#include "adc_monitor.h"

#if V_PMB_INIT == 1
int m_tempWarning;
unsigned int m_pGoodOn;
unsigned int m_pGoodOff;
#endif
unsigned char m_softWarningCheckEn=0;
unsigned char filterFilled=0;
unsigned char m_selectedAdcChanel = 0;

void initADCVal()
{
    filterFilled=0; //Used to wait the average has been filled with values
#if V_PMB_INIT == 1
    m_pGoodOn = GET_MANTISSA(POWER_GOOD_ON)*4/5;
    m_pGoodOff = GET_MANTISSA(POWER_GOOD_OFF)*4/5;
    m_tempWarning = GET_MANTISSA(OT_WARN_LIMIT);
#endif
}


/********************************************************************
 * Function:        	runAdcMonitor
 *
 * Overview:        	ADC monitoring
 *******************************************************************/
void runAdcMonitor()
{
        //If the ADC is available
        if (!GO_nDONE)//not busy? (+sppReq priority)
        {
            //Part 1: Check the last converted value
            int sum = CHAR_CONCAT_TO_UINT(ADRESH, ADRESL);
            //Do the averaging (AVG index 0 -> avg val)
#if AVG_SIZE > 1
            for (int i = 0; i < AVG_SIZE-1; i++)
            {
                int valTmp = m_adcValueList[m_selectedAdcChanel][i+1];
                m_adcValueList[m_selectedAdcChanel][i] = valTmp;
                sum += valTmp;
            }
#endif
            sum += m_adcValueList[m_selectedAdcChanel][0];
            sum = sum/(AVG_SIZE+1);
            //Place the AVG result in index 0
            m_adcValueList[m_selectedAdcChanel][AVG_SIZE-1] = sum;

            //Check that the AVG fliter is full (+margin)
            if (filterFilled<=AVG_SIZE*ADC_MON_CHAN_SIZE*4)
                filterFilled++;

            if (!m_softWarningCheckEn)
                filterFilled=0;

#if V_PMB_INIT == 1
            //Part 2: Check read values (Power Good, Over temperature)
            if (m_softWarningCheckEn && filterFilled>AVG_SIZE*ADC_MON_CHAN_SIZE*4)
            {
                switch (m_selectedAdcChanel)
                {
                    case ADC_VAL_INT_VREG: //output voltage channel
                    {
                        TRISA6=0;
                        if (m_adcValueList[ADC_VAL_INT_VREG][0] < m_pGoodOff)
                        {
                            STATUS_WORD[1] |= STATUS_N_PGOOD;
                            GPA6=1;
                        }else if (m_adcValueList[ADC_VAL_INT_VREG][0] > m_pGoodOn)
                        {
                            STATUS_WORD[1] &= ~STATUS_N_PGOOD;
                            GPA6=0;
                        }
                    }
                    break;

                    case ADC_VAL_TMP_ANA :
                    {
                        sum = m_adcValueList[ADC_VAL_TMP_ANA][0];
                        if ( ((sum*2/5) - 142) > m_tempWarning)
                            STATUS_WORD[0] |= STATUS_TEMP_WARNING;
                    }
                    break;
                }
            }
#endif
            //Increase the selected chanel to that one has to be converted
            //and launch a new conversion.
            m_selectedAdcChanel = (m_selectedAdcChanel + 1) % ADC_MON_CHAN_SIZE;
            SET_ADC_CHANEL(m_adcChannelMonitorList[m_selectedAdcChanel]);
            GO_nDONE = 1;
        
        }
}

/********************************************************************
 * Function:        	adcDirectRead
 *
 * Input:        	char channel: Selected channel (See part datasheet or
 *                      V_DEF_MCP19111.h)
 * Output:              Value from th ADC
 *
 * Overview:        	ADC read function. This function is blocking the current
 *                      program until a value from the ADC has been readed
 *******************************************************************/
int adcDirectRead(char channel) {
    while (GO_nDONE); //Not allready is used?
    SET_ADC_CHANEL(channel); //Set channel
    NOP();
    NOP();
    NOP();
    NOP();
    GO_nDONE = 1; //Start conversion
    while (GO_nDONE); //Wait conversion
    return CHAR_CONCAT_TO_UINT(ADRESH, ADRESL); //Read value
}


