/********************************************************************
 * FileName:        commands.h
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/

#ifndef COMMANDS_H
#define COMMANDS_H
#include "V_DEF_MCP19111.h"
//-----------------------------------------------
// PMBus Protocols
//-----------------------------------------------
#define NO_FLASH 0xFF

typedef struct {
    unsigned char inFlash;
    unsigned char nrBytesWr;
    unsigned char nrBytesRd;
    unsigned char * ptrCommandData;
    unsigned char codeNr;
} commandInfo;

typedef struct {
    const bank1 unsigned char nrBytesRd;
    const unsigned char * ptrCommandData;
    const bank1 unsigned char codeNr;
} commandInfoFixed;


#define NR_COMMANDS 29
#define NR_COMMANDS_FIXED 6
#define UNSUPPORTED_CMD_CODE 0xff
#define MAX_B 2

#define OPERATION_COMMAND 0X01
#define ON_OFF_CONFIG_COMMAND 0X02
#define CLEAR_FAULTS_COMMAND 0X03
#define STORE_DEFAULT_ALL_COMMAND 0X11
#define RESTORE_DEFAULT_ALL_COMMAND 0X12
#define VOUT_COMMAND_COMMAND 0X15
#define VOUT_MODE_COMMAND 0X20
#define VOUT_TRIM_COMMAND 0X22
#define VOUT_MARGIN_HIGH_COMMAND 0X25
#define VOUT_MARGIN_LOW_COMMAND 0X26
#define MAX_DUTY_COMMAND 0X32
#define FREQUENCY_SWITCH_COMMAND 0X33
#define VIN_ON_COMMAND 0X35
#define VIN_OFF_COMMAND 0X36
#define INTERLEAVE_COMMAND 0X37
#define VOUT_OV_FAULT_LIMIT_COMMAND 0X40
#define VOUT_OV_FAULT_RESPONSE_COMMAND 0X41
#define VOUT_UV_FAULT_LIMIT_COMMAND 0X44
#define VOUT_UV_FAULT_RESPONSE_COMMAND 0X45
#define IOUT_OC_FAULT_LIMIT_COMMAND 0X46
#define IOUT_OC_FAULT_RESPONSE_COMMAND 0X47
#define OT_WARN_LIMIT_COMMAND 0X51
#define POWER_GOOD_ON_COMMAND 0X5E
#define POWER_GOOD_OFF_COMMAND 0X5F
#define TON_RISE_COMMAND 0X61
#define STATUS_WORD_COMMAND 0X79
#define STATUS_VOUT_COMMAND 0X7A
#define STATUS_CML_COMMAND 0X7E
#define READ_VIN_COMMAND 0X88
#define READ_VOUT_COMMAND 0X8B
#define READ_IOUT_COMMAND 0X8C
#define READ_TEMPERATURE_1_COMMAND 0X8D
#define READ_DUTY_CYCLE_COMMAND 0X94
#define PMBUS_REVISION_COMMAND 0X98
#define REVISION_COMMAND 0XFC

#if V_PMB_INIT == 1
unsigned char bufferTmp[MAX_B + 1];


const bank1 unsigned char VOUT_MODE[1]={V_VOUT_MODE};
bank1 unsigned char STATUS_VOUT[1]={V_STATUS_VOUT};
bank1 unsigned char STATUS_CML[1]={V_STATUS_CML};
bank1 unsigned char VOUT_TRIM[2]={V_VOUT_TRIM_L, V_VOUT_TRIM_H };
bank1 unsigned char MAX_DUTY[2]={V_MAX_DUTY_L, V_MAX_DUTY_H };
bank1 unsigned char FREQUENCY_SWITCH[2]={V_FREQUENCY_SWITCH_L, V_FREQUENCY_SWITCH_H };
bank1 unsigned char INTERLEAVE[2]={V_INTERLEAVE_L, V_INTERLEAVE_H };
bank1 unsigned char VOUT_OV_FAULT_LIMIT[2]={V_VOUT_OV_FAULT_LIMIT_L, V_VOUT_OV_FAULT_LIMIT_H };
bank1 unsigned char VOUT_UV_FAULT_LIMIT[2]={V_VOUT_UV_FAULT_LIMIT_L, V_VOUT_UV_FAULT_LIMIT_H };
bank1 unsigned char IOUT_OC_FAULT_LIMIT[2]={V_IOUT_OC_FAULT_LIMIT_L, V_IOUT_OC_FAULT_LIMIT_H };
bank1 unsigned char OT_WARN_LIMIT[2]={V_OT_WARN_LIMIT_L, V_OT_WARN_LIMIT_H };
bank1 unsigned char POWER_GOOD_ON[2]={V_POWER_GOOD_ON_L, V_POWER_GOOD_ON_H };
bank1 unsigned char POWER_GOOD_OFF[2]={V_POWER_GOOD_OFF_L, V_POWER_GOOD_OFF_H };
bank1 unsigned char TON_RISE[2]={V_TON_RISE_L, V_TON_RISE_H };
bank1 unsigned char READ_VIN[2]={V_READ_VIN_L, V_READ_VIN_H };
bank1 unsigned char READ_VOUT[2]={V_READ_VOUT_L, V_READ_VOUT_H };
bank1 unsigned char READ_IOUT[2]={V_READ_IOUT_L, V_READ_IOUT_H };
bank1 unsigned char READ_TEMPERATURE_1[2]={V_READ_TEMPERATURE_1_L, V_READ_TEMPERATURE_1_H };
bank1 unsigned char READ_DUTY_CYCLE[2]={V_READ_DUTY_CYCLE_L, V_READ_DUTY_CYCLE_H };
const bank1 unsigned char PMBUS_REVISION[1]={V_PMBUS_REVISION};
const bank1 unsigned char REVISION[2]={V_REVISION_L, V_REVISION_H };
bank1 unsigned char VOUT_COMMAND[2]={V_VOUT_COMMAND_L, V_VOUT_COMMAND_H };
bank1 unsigned char VOUT_MARGIN_HIGH[2]={V_VOUT_MARGIN_HIGH_L, V_VOUT_MARGIN_HIGH_H };
bank1 unsigned char VOUT_MARGIN_LOW[2]={V_VOUT_MARGIN_LOW_L, V_VOUT_MARGIN_LOW_H };
bank1 unsigned char DUMMY[1];
#endif

bank1 unsigned char OPERATION[1]={V_OPERATION};
bank1 unsigned char ON_OFF_CONFIG[1]={V_ON_OFF_CONFIG};
bank1 unsigned char VIN_ON[2]={V_VIN_ON_L, V_VIN_ON_H };
bank1 unsigned char VIN_OFF[2]={V_VIN_OFF_L, V_VIN_OFF_H };
bank1 unsigned char VOUT_OV_FAULT_RESPONSE[1]={V_VOUT_OV_FAULT_RESPONSE};
bank1 unsigned char VOUT_UV_FAULT_RESPONSE[1]={V_VOUT_UV_FAULT_RESPONSE};
bank1 unsigned char IOUT_OC_FAULT_RESPONSE[1]={V_IOUT_OC_FAULT_RESPONSE};
bank1 unsigned char STATUS_WORD[2]={V_STATUS_WORD_L, V_STATUS_WORD_H };




#if V_PMB_INIT == 1

const commandInfo matrix[NR_COMMANDS] = {
   //Flash Index; Nr_wr; Nr_rd; Command_ram_index; Code
{NO_FLASH, 1, 1,OPERATION,OPERATION_COMMAND},
{0, 1, 1,ON_OFF_CONFIG,ON_OFF_CONFIG_COMMAND},
{1, 2, 2,VOUT_COMMAND,VOUT_COMMAND_COMMAND},
{3, 2, 2,VOUT_TRIM,VOUT_TRIM_COMMAND},
{5, 2, 2,VOUT_MARGIN_HIGH,VOUT_MARGIN_HIGH_COMMAND},
{7, 2, 2,VOUT_MARGIN_LOW,VOUT_MARGIN_LOW_COMMAND},
{9, 2, 2,MAX_DUTY,MAX_DUTY_COMMAND},
{11, 2, 2,FREQUENCY_SWITCH,FREQUENCY_SWITCH_COMMAND},
{13, 2, 2,VIN_ON,VIN_ON_COMMAND},
{15, 2, 2,VIN_OFF,VIN_OFF_COMMAND},
{17, 2, 2,INTERLEAVE,INTERLEAVE_COMMAND},
{19, 2, 2,VOUT_OV_FAULT_LIMIT,VOUT_OV_FAULT_LIMIT_COMMAND},
{21, 1, 1,VOUT_OV_FAULT_RESPONSE,VOUT_OV_FAULT_RESPONSE_COMMAND},
{22, 2, 2,VOUT_UV_FAULT_LIMIT,VOUT_UV_FAULT_LIMIT_COMMAND},
{24, 1, 1,VOUT_UV_FAULT_RESPONSE,VOUT_UV_FAULT_RESPONSE_COMMAND},
{25, 2, 2,IOUT_OC_FAULT_LIMIT,IOUT_OC_FAULT_LIMIT_COMMAND},
{27, 1, 1,IOUT_OC_FAULT_RESPONSE,IOUT_OC_FAULT_RESPONSE_COMMAND},
{28, 2, 2,OT_WARN_LIMIT,OT_WARN_LIMIT_COMMAND},
{30, 2, 2,POWER_GOOD_ON,POWER_GOOD_ON_COMMAND},
{32, 2, 2,POWER_GOOD_OFF,POWER_GOOD_OFF_COMMAND},
{34, 2, 2,TON_RISE,TON_RISE_COMMAND},
{NO_FLASH, 0, 2,STATUS_WORD,STATUS_WORD_COMMAND},
{NO_FLASH, 0, 1,STATUS_VOUT,STATUS_VOUT_COMMAND},
{NO_FLASH, 0, 1,STATUS_CML,STATUS_CML_COMMAND},
{NO_FLASH, 2, 2,READ_VIN,READ_VIN_COMMAND},
{NO_FLASH, 2, 2,READ_VOUT,READ_VOUT_COMMAND},
{NO_FLASH, 2, 2,READ_IOUT,READ_IOUT_COMMAND},
{NO_FLASH, 2, 2,READ_TEMPERATURE_1,READ_TEMPERATURE_1_COMMAND},
{NO_FLASH, 2, 2,READ_DUTY_CYCLE,READ_DUTY_CYCLE_COMMAND}
};

const commandInfoFixed matrixFixed[NR_COMMANDS] = {
   //Nr_rd; Command_ram_index; Code
{0,DUMMY,CLEAR_FAULTS_COMMAND},
{0,DUMMY,STORE_DEFAULT_ALL_COMMAND},
{0,DUMMY,RESTORE_DEFAULT_ALL_COMMAND},
{1,VOUT_MODE,VOUT_MODE_COMMAND},
{1,PMBUS_REVISION,PMBUS_REVISION_COMMAND},
{2,REVISION,REVISION_COMMAND}
};

#endif

#endif /* PMBUS_HANDLER_H */
