/********************************************************************
 * FileName:        init_part.c
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                02.03.2013  Initial release
 ********************************************************************/


#include "init_part.h"

void initPart()
{
    //INIT CALIBRATION REGISTERS:
    unsigned char calBuf[2];
    readFlash(DEF_CAL_DOV_F, calBuf, 1);
    DOVCAL = calBuf[1];
    OSCCAL = calBuf[0];
    readFlash(DEF_CAL_VRO_BGR, calBuf, 1);
    VROCAL = calBuf[1];
    BGRCAL = calBuf[0];
    readFlash(DEF_CAL_TTA_ZRO, calBuf, 1);
    TTACAL = calBuf[1];
    ZROCAL = calBuf[0];


    //LOAD PROTECTION REGISTERS VALUE (but not enabled here)
#if V_PMB_INIT == 0
    OUVCON = V_OUVCON;
    OOVCON = V_OOVCON;
#endif
    OCCON = V_OCCON;
    VINLVL = V_VINLVL;

    //OPTION
    OPTION_REG = DEF_OPTION_REG;
    //Timer 1 config
    T1CON = DEF_T1CON;

    // SET SWITCHING FREQUENCY
#if V_PMB_INIT == 0
    PWMRL = V_PWMRL;
    PWMPHL = V_PWMPHL;
    PR2 = V_PR2;
#endif
    T2CON = V_T2CON;

    // SWITCHING DEAD TIME SETTING
    DEADCON = V_DEADCON;

    // COMPENSATION SETTING
    CMPZCON = V_CMPZCON;
    SLPCRCON = V_SLPCRCON;
    SLVGNCON = V_SLVGNCON;

    // CURRENT SENSE SETTING
    CSGSCON = V_CSGSCON;
    VZCCON = V_VZCCON;
    CSDGCON = V_CSDGCON;

    // ADC SETTING
    ADCON0 = DEF_ADCON0;
    ADCON1 = DEF_ADCON1;


    // SET OUTPUT VOLTAGE
    OVFCON = V_OVFCON;
    //Wait for VIN UV gone
    PIR2bits.VINIF = 0;
    while (VINIF == 1)
        PIR2bits.VINIF = 0;

    //(VOUT UV protection should not be enable during soft start if enabled)
    // ENABLE PERIPHERAL
    PE1 = V_PE1;
    BUFFCON = V_BUFFCON;
    ABECON = V_ABECON & NOT_OV_UV_MASK; //set later OOV OUV when OV set
    ATSTCON = V_ATSTCON | 1;

    //Init IO CONFIG
    //Set all pins as input, expect GPA 0 and 2 which are as output and GPA1 which depends of the output mode (SLAVE/MASTER)
    const unsigned char MLTPH = V_BUFFCON >> 5;
    if ((MLTPH == 2) || (MLTPH == 4))
        TRISGPA = 0xFA; //Slave output mode
    else
        TRISGPA = 0xF8; //Master/stand alone output mode

    TRISGPB = 0xFF;
    //Set all pin as digital IO
    ANSELA = 0; 
    ANSELB = 0;
    WPUGPA = 0x0;
    WPUGPB = 0x0;

    //IO pin special setting
    EN_PIN_WPU = 1; //weak pull up enable
    EN_PIN_IO = 1; //input state
    EN_PIN = 0;
    EN_PIN_INT_EN = 1; //interrupt enable

    SMBALERT_PIN = 0;
    SMBALERT_PIN_IO = 1;

#if V_PMB_INIT == 1
#ifdef I2C_ADDR_PIN_EN //Use a pin voltage to set the I2C Address
    I2C_ADDR_PIN_IO = 1;
    I2C_ADDR_PIN_AN = 1;
    //This will read the value on GPB2 using the ADC (10bits) and the I divide by 4 (8bits)
    m_slaveAddr = (adcDirectRead(ADC_GPA0 + V_NUM_I2C_ADDR_PIN))>>2;
    //mask for write address only + remove one more LSB for repeatability
    m_slaveAddr = m_slaveAddr & 0xFC;

#else
     m_slaveAddr = V_SLAVE_ADDR;
#endif //I2C_ADDR_PIN_EN
#endif //V_PMB_INIT


    //CLEAR INTERRUPT FLAGS
    PIR1 = 0x00;
    PIR2 = 0x00;

    // INTERRUPT SETTING
    PIE1 = DEF_PIE1;
    INTCON = DEF_INTCON;


}

