 /********************************************************************
 * FileName:        pmbus_core.h
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/


#include "pmbus_core.h"
#if V_PMB_INIT == 1
unsigned char m_counter = 0;
unsigned char m_byteCRC = 0;
unsigned char m_dumpSSPBUF = 0;
unsigned char m_byteNr = 0;
unsigned char m_arrayIndex=0;
bit m_useFixed=0;
int i2cTimeout = 0;

/********************************************************************
 * Function:        	initPMBusStack
 *
 * PreCondition: 	MSSP module
 *
 * Overview:		Function that configures Mcu resources for
 *			the stack functionality. User has to be defined
 *			it for either MSSP or SSP.
 *******************************************************************/

void initPMBusStack() {

    SSPCON1 = 0; // Slave mode, 7bit addr, S/P interrputs
    OPTION_REG = DEF_OPTION_REG; // TMR0 in couner mode
    SSPSTAT = DEF_SSPSTAT; // 100Khz
    SSPADD = m_slaveAddr;
    SSPADD2 = SLAVE_ALERT_DIS_ADDR;
    SSPCON1 = DEF_SSPCON1; // Slave mode, 7bit addr, S/P interrputs
    SSPCON2 = DEF_SSPCON2; // enable general call
    SSPCON3 = DEF_SSPCON3; // enable general call
    SSPIF = 0x0; // clear interrupt flag
    clearFaults();
}


/********************************************************************
 * Function:        	codeToIndex
 *
 * PreCondition: 	PMBUS stack
 *
 * Overview:		Function that return the RAM address of the PMBUS
 *                      commands records, using the PMBus code as input
 *******************************************************************/
unsigned char codeToIndex(unsigned char code) {
    switch(code)
    {
        case OPERATION_COMMAND: return 0;
        case ON_OFF_CONFIG_COMMAND: return 1;
        case VOUT_COMMAND_COMMAND: return 2;
        case VOUT_TRIM_COMMAND: return 3;
        case VOUT_MARGIN_HIGH_COMMAND: return 4;
        case VOUT_MARGIN_LOW_COMMAND: return 5;
        case MAX_DUTY_COMMAND: return 6;
        case FREQUENCY_SWITCH_COMMAND: return 7;
        case VIN_ON_COMMAND: return 8;
        case VIN_OFF_COMMAND: return 9;
        case INTERLEAVE_COMMAND: return 10;
        case VOUT_OV_FAULT_LIMIT_COMMAND: return 11;
        case VOUT_OV_FAULT_RESPONSE_COMMAND: return 12;
        case VOUT_UV_FAULT_LIMIT_COMMAND: return 13;
        case VOUT_UV_FAULT_RESPONSE_COMMAND: return 14;
        case IOUT_OC_FAULT_LIMIT_COMMAND: return 15;
        case IOUT_OC_FAULT_RESPONSE_COMMAND: return 16;
        case OT_WARN_LIMIT_COMMAND: return 17;
        case POWER_GOOD_ON_COMMAND: return 18;
        case POWER_GOOD_OFF_COMMAND: return 19;
        case TON_RISE_COMMAND: return 20;
        case STATUS_WORD_COMMAND: return 21;
        case STATUS_VOUT_COMMAND: return 22;
        case STATUS_CML_COMMAND: return 23;
        case READ_VIN_COMMAND: return 24;
        case READ_VOUT_COMMAND: return 25;
        case READ_IOUT_COMMAND: return 26;
        case READ_TEMPERATURE_1_COMMAND: return 27;
        case READ_DUTY_CYCLE_COMMAND: return 28;
        default : return UNSUPPORTED_CMD_CODE;
    }
}

/********************************************************************
 * Function:        	codeToIndexFixed
 *
 * PreCondition: 	PMBUS stack
 *
 * Overview:		Function that return the RAM address of the PMBUS
 *                      fixed commands records, using the PMBus code as input
 *******************************************************************/
unsigned char codeToIndexFixed(unsigned char code) {
    switch(code)
    {
        case CLEAR_FAULTS_COMMAND: return 0;
        case STORE_DEFAULT_ALL_COMMAND: return 1;
        case RESTORE_DEFAULT_ALL_COMMAND: return 2;
        case VOUT_MODE_COMMAND: return 3;
        case PMBUS_REVISION_COMMAND: return 4;
        case REVISION_COMMAND: return 5;
        default : return UNSUPPORTED_CMD_CODE;
    }
}


/********************************************************************
 * Function:        	applyAllPMBSettings
 *
 * Overview:		Fonction the will apply all PMBus commands with the
 *                      current value. The loadFromFlash parameter allows to
 *                      load the value from flash before applying the commands.
 *******************************************************************/
void applyAllPMBSettings(char loadFromFlash)
{
    if (loadFromFlash)
    {
        m_codeCMD = RESTORE_DEFAULT_ALL_COMMAND;
        pmbusHostWrite();
    }

    for (int i=0; i<NR_COMMANDS-1; i++)
    {
        m_codeCMD = matrix[i].codeNr;
        pmbusHostWrite();
    }
}

/********************************************************************
 * Function:        transContentFaultHandler
 *
 * Input:           Token to determine the fault nature.
 *
 * Overview:        This function modifies stack flags
 *                  and STATUS registers as an action to a present
 *                  data transmission or data content fault.
 *******************************************************************/
void transContentFaultHandler(char errNum) {

    i2c_flags.ready_to_copy = 0;
    i2c_flags.transaction_on = 0;

    switch (errNum) {
        case 1:
            STATUS_WORD[0] |= STATUS_CML_FAULT; //Corrupted data(Pec byte error)
            STATUS_CML[0] |= COM_CRC_FAULT;
            break;
        case 2:
            STATUS_WORD[0] |= STATUS_CML_FAULT; //Sending/reading too few bits	OR
            STATUS_CML[0] |= COM_OTHER_COM_FAULT; //Reading too many bytes
            break;
        case 3:
            STATUS_WORD[0] |= STATUS_CML_FAULT; //Host sends too many bytes
            STATUS_CML[0] |= COM_DATA_FAULT;
            break;
//        case 4:
//            STATUS_WORD[0] |= STATUS_BUSY; //Device busy
//            break;
//        case 5:
//            STATUS_WORD[0] |= STATUS_CML_FAULT; //Improper address bit in the address byte
//            STATUS_CML[0] |= COM_OTHER_COM_FAULT;
//            break;
        case 6:
            STATUS_WORD[0] |= STATUS_CML_FAULT; //Unsupported command code
            STATUS_CML[0] |= COM_COMMAND_FAULT;
            break;
    }

}

/********************************************************************
 * Function:		writeData
 *
 * PreCondition: 	The PMBUS slave is in a position which requires it
 *			to put data on the bus.
 *
 * Input:        	Data to be put on the bus

 * Overview:		This function receives data as a parameter and fills
 *			the I2C SSPBUF. At the end it disables the clock streching
 *			mechanism.
 *******************************************************************/
void writeData(unsigned char data) {
    do {
        WCOL = 0;
        SSPBUF = data; // need further parameters
    } while (WCOL);
    CKP = 1;
}

/********************************************************************
 * Function:			basicCrc
 *
 * PreCondition: 	None
 *
 * Input:        	The parameters are the result of the previous
 *					calculation and the next byte to be added to the
 *					calculus
 *
 * Output:     		Result of m_byteCRC8 calculus
 *
 * Side Effects: 	None
 *
 * Overview:			Basic m_byteCRC8 calculus
 *
 * Note:        		None
 *******************************************************************/
unsigned char basicCrc(unsigned char remainder, unsigned char byte) {
    const char POLY = 0x7;
    const char WIDTH = 8;
    const char MSB = (1 << 7);
    remainder ^= byte;
    
    for (char i = WIDTH; i > 0; --i) {
        if (remainder & MSB) 
            remainder = (remainder << 1) ^ POLY;
        else
            remainder = (remainder << 1);
    }
    return remainder;
}


/********************************************************************
 * Function:		hostReadsData
 *
 * PreCondition: 	None
 *
 * Input:        	None
 *
 * Output:     	None
 *
 * Side Effects: 	None
 *
 * Overview:          As the host keeps ACKing bytes sent by the slave,
 *                    this function takes valid data and puts it on the
 *			bus, checking at the same time for the validity
 *			of the host's request for more data bytes
 *
 * Note:        		None
 *******************************************************************/
void hostReadsData() {
    unsigned char toSend=0;

    if (m_counter < m_byteNr) //read data
    {
        if (m_useFixed)
            toSend = *(matrixFixed[m_arrayIndex].ptrCommandData+m_counter);
        else
            toSend = *(matrix[m_arrayIndex].ptrCommandData+m_counter);
        m_byteCRC = basicCrc(m_byteCRC, toSend); //do CRC-8 on COMMAND code
        writeData(toSend);
    }
    else if (m_counter == m_byteNr) //should be PEC data
        writeData(m_byteCRC);
    else if (m_counter == (m_byteNr+1)) //Closing connection (Really?!?!?!?)
          i2c_flags.transaction_on = 0;
    else
        transContentFaultHandler(2); //unexptected data

    m_counter++; //inc MRD state counter
}

/********************************************************************
 * Function:		i2cSlaveHandler
 *
 * PreCondition: 	None
 *
 * Input:        	None
 *
 * Output:     		None
 *
 * Side Effects: 	None
 *
 * Overview:            Function called from the main application as a source
 *			of interrupt from the MSSP (SSPIF flag)module has been
 *			detected. This function achieves the functionality of
 *			the MSSP module as a I2C slave 7bit address, S/P interrupts
 *
 * Note:        	None
 *******************************************************************/
void i2cSlaveHandler() {

    char token = 0;
    token = SSPSTAT & STAT_MASK; //obtain current state
    m_dumpSSPBUF = SSPBUF;

    switch (token) {
        case MASTER_WRITE_ADDRESS: //MASTER WRITES ADDRESS STATE
        {
            m_byteCRC = 0;
            m_byteCRC = basicCrc(m_byteCRC, m_slaveAddr);
            bufferTmp[0] = 0; //reset nr of bytes in buffer

            i2c_flags.transaction_on = 1;
            m_byteNr = 0;
            m_counter = 0;
            break;
        }
        case MASTER_WRITE_DATA: //MASTER WRITES DATA STATE
            if (i2c_flags.transaction_on==1) //transaction in progress?
            {
                //-------------------------------------------
                //HOST IS SENDING DATA
                //-------------------------------------------

                    if (m_counter==0)//first byte, host send command
                    {
                        m_codeCMD = m_dumpSSPBUF;
                        m_byteCRC = basicCrc(m_byteCRC, m_codeCMD); //do CRC-8 on COMMAND code
                        m_arrayIndex = codeToIndex(m_codeCMD); //get Matrix index
                        if (m_arrayIndex==UNSUPPORTED_CMD_CODE) //CMD not supported?
                        {
                            m_useFixed=1; //Verify the second Matrix
                            m_arrayIndex = codeToIndexFixed(m_codeCMD); //is it a fixed CMD supported?
                            if (m_arrayIndex==UNSUPPORTED_CMD_CODE) //CMD not supported?
                            {
                                bufferTmp[0]=0;
                                transContentFaultHandler(6);
                            }
                        } else
                        {
                            m_useFixed=0;
                            m_byteNr = matrix[m_arrayIndex].nrBytesWr;
                            bufferTmp[0]=m_byteNr;
                        }
                    } else if (m_counter <= m_byteNr) //store data
                    {
                        bufferTmp[m_counter] = m_dumpSSPBUF;
                        m_byteCRC = basicCrc(m_byteCRC, m_dumpSSPBUF); //do CRC-8 on COMMAND code
                    } else if (m_counter == (m_byteNr+1)) //should be PEC data
                    {
                        if (m_byteCRC != m_dumpSSPBUF) {
                            transContentFaultHandler(1); //wrong pec byte
                        } else {
                            i2c_flags.transaction_on = 0;//transmission ended. Data can be used
                            for (unsigned char i = 0; i < bufferTmp[0]; i++) {
                                *((matrix[m_arrayIndex].ptrCommandData) + i) = bufferTmp[i + 1];
                            }
                            i2c_flags.ready_to_copy = 1; //main loop can act on new value.
                        }
                    } else
                        transContentFaultHandler(2); //unexptected data

                    m_counter++;
                //-------------------------------------------
                //END OF HOST DATA HANDLING
                //-------------------------------------------

            }
            else
                transContentFaultHandler(2);
            break;

        case MASTER_READ_ADDRESS: //MASTER READS ADDRESS STATE
        {
            m_counter = 0;
            if (m_dumpSSPBUF != V_SLAVE_ALERT_ADDR)
            {
                i2c_flags.transaction_on = 1;
                m_byteCRC = basicCrc(m_byteCRC, m_slaveAddr|1);
                
                if (m_useFixed)
                    m_byteNr = matrixFixed[m_arrayIndex].nrBytesRd;
                else
                {
                    m_byteNr = matrix[m_arrayIndex].nrBytesRd;
                    pmbusHostRead(matrix[m_arrayIndex].codeNr, matrix[m_arrayIndex].ptrCommandData);
                }
                hostReadsData();
            } else //resp ARA..
            {
                i2c_flags.ara_transaction_on = 1;
                writeData(m_slaveAddr);
                m_byteCRC = 0;
                m_byteCRC = basicCrc(m_byteCRC, V_SLAVE_ALERT_ADDR);
                m_byteCRC = basicCrc(m_byteCRC, m_slaveAddr);
            }
            break;
        }
        case MASTER_READ_DATA: //MASTER READS DATA STATE
            if ((i2c_flags.transaction_on==1))
            { //transaction in progress?
                hostReadsData();
            }
            else if (i2c_flags.ara_transaction_on==1)
            {
                writeData(m_byteCRC);
                i2c_flags.ara_transaction_on=0;
            }
            else
            {
                writeData(0xff); //or improper address bit in the address byte
                transContentFaultHandler(2);
            }
            break;
    }
    if(SSPOV) //if overflow, clear flag. this is done here to allow
        SSPOV=0;

    SSPIF = 0x0;
    SSPEN = 1;
    CKP = 1; //release clock
}

/********************************************************************
 * Function:        	clearFaults
 *
 * PreCondition: 	Appropriate action generated by the host(e.g. CLEAR_FAULTS
 *			COMMAND received)
 *
 * Overview:        	This function is supplied to the application as a mean to clear
 *			the STATUS registers following the appropriate action generated
 *			by the host.
 *
 * Note:        		Must be place in an application file
 *******************************************************************/
void clearFaults(void) {

    STATUS_WORD[0] = STATUS_WORD[0] & STATUS_OFF; //STATUS_BYTE is the first byte of STATUS_WORD
    STATUS_WORD[1] = 0;
    STATUS_VOUT[0] = 0;
    STATUS_CML[0] = 0;
}

/********************************************************************
 * Function:        	PMBusCheckTimeout
 *
 * PreCondition: 	The main loop or a timer has to call it periodically
 *
 * Overview:        	This function check if the SDA line timeout
 *
 * Note:        	Must be place in an application file
 *******************************************************************/
void PMBusCheckTimeout(void) {
    if (!GPB0) { // buffer ready to be copied?
        i2cTimeout++; // clear flag
        if (i2cTimeout>I2C_TIME_OUT)
        {
            i2cTimeout=0;
            transContentFaultHandler(2);
            initPMBusStack(); // call copy function
        }
    } else
    {
        i2cTimeout=0;
    }
}


#endif

