 /********************************************************************
 * FileName:        pmbus_handler.h
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/

#ifndef PMBUS_CORE_H
#define	PMBUS_CORE_H

#ifdef	__cplusplus
extern "C" {
#endif

#include "adc_monitor.h"
#include "pmbus_handler.h"
#include "math.h"
#include "macro.h"
#include "V_DEF_MCP19111.h"
#include <pic.h>

//----STATUS REG DEFINITION:
//Status LSB Word PMBus Reg
#define STATUS_BUSY         0x80
#define STATUS_OFF          0x40
#define STATUS_OV_OV_FAULT  0x20
#define STATUS_OC_FAULT     0x10
#define STATUS_VIN_FAULT    0x08
#define STATUS_TEMP_WARNING 0x04
#define STATUS_CML_FAULT    0x02
#define STATUS_OTHER_ERR    0x01
//Status MSB Word PMBus Reg
#define STATUS_OV_ERR       0x80
#define STATUS_OP_ERR       0x40
#define STATUS_VIN_ERR      0x20
#define STATUS_N_PGOOD      0x08
#define STATUS_ANY_OTHER_FAULT 0x01

//Status VOUT PMBus Reg
#define OV_OV_FAULT         0x80
#define OV_OV_WARNING       0x40
#define OV_UV_WARNING       0x20
#define OV_UV_FAULT         0x10
//Status IOUT PMBus Reg
#define OC_FAULT            0x80
//Status INPUT PMBus Reg
#define VIN_OV_WARNING      0x40
#define VIN_UV_WARNING      0x20
#define VIN_UV_FAULT        0x10
//Status Temp PMBus Reg
#define OT_WARNING          0x40
#define UT_WARNING          0x20
//Status CML PMBus Reg
#define COM_COMMAND_FAULT   0x80
#define COM_DATA_FAULT      0x40
#define COM_CRC_FAULT       0x20
#define COM_MEM_FAULT       0x10
#define COM_MCU_FAULT       0x08
#define COM_OTHER_COM_FAULT 0x02
#define COM_OTHER_MEM_FAULT 0x01
//Status IOUT PMBus Reg
#define IOUT_OC_FAULT       0x07
#define IOUT_OC_WARNING     0x05

#define ON_OFF_ALWAYS_ON_B    0x10
#define ON_OFF_I2C_ON       0x08
#define ON_OFF_PIN_ON       0x04
#define ON_OFF_PIN_POLARITY 0x02

#define OPERATION_ENABLE    0x80
#define OPERATION_NORMAL      0b110000
#define OPERATION_MARGIN_HIGH 0b100000
#define OPERATION_MARGIN_LOW  0b10000
#define OPERATION_MARGIN_HIGH 0b100000
#define OPERATION_MARGIN_MASK  0b110000
#define OPERATION_IGNORE_FAULT  0b0100

#define RESP_TRIED_MASK     0x38
#define GET_RESP_TRIED(x)   x & RESP_TRIED_MASK
#define SUB_RESP_TRIED(x)   x=x-8
#define GET_RESP_MODE(x)    x>>6

#define RESTART_IF_FAULT_AGAIN 1
#define DIRECT_RESTART_IF_FAULT 2
#define DIRECT_STOP 3

#define I2C_TIME_OUT 0x7F

#define MASTER_WRITE_ADDRESS 0x1		//Master Writes Address
#define MASTER_WRITE_DATA 0x21				//Master Writes Data
#define STAT_MASK 0x25 //I2C states mask
#define MASTER_READ_ADDRESS 0x5				//Master Reads Address
#define MASTER_READ_DATA 0x24				//Master Reads Data

#define VIN_FIXED_EXP 0xF0 //-2
#define IOUT_FIXED_EXP 0xF8 //-1
#define TEMP_FIXED_EXP 0x0


#if V_PMB_INIT == 1

unsigned char basicCrc(unsigned char remainder, unsigned char byte);
void i2cSlaveHandler();
void clearFaults(void);
void applyAllPMBSettings(char loadFromFlash);
void initPMBusStack(void);
void transContentFaultHandler(char token);
void PMBusCheckTimeout(void);

unsigned char m_codeCMD;
unsigned char m_slaveAddr;


struct {
    unsigned char ready_to_copy:1;
    unsigned char device_busy:1;
    unsigned char transaction_on:1;
    unsigned char ara_transaction_on:1;
    unsigned char apply_all_req:1;
}
i2c_flags = {0};

//extern unsigned int m_adcValueList[ADC_MON_CHAN_SIZE][AVG_SIZE];
extern unsigned char m_OVCCON_target;
extern unsigned char m_TMR1H;
extern unsigned char m_TMR1L;

#endif

#ifdef	__cplusplus
}
#endif

#endif	/* PMBUS_CORE_H */