#include <plib.h>
#include <p32xxxx.h>
#include ".\..\APP2013TM_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 20 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC3 , PGD3 , Disable SOSC
// Disable JTAG , IO One Way OFF , PMDL One Way OFF
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_20 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_1
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx3 , FSOSCEN = OFF
#pragma config JTAGEN = OFF , IOL1WAY = OFF , PMDL1WAY = OFF
#pragma config UPLLIDIV = DIV_2 , UPLLEN = ON , FUSBIDIO = OFF

#define SystemFrequency		80000000L
#define PBFrequency		SystemFrequency / 8
#define Timer1Tick		( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		10
#define UARTChannel UART2	/* UART2 for APP1632 */
// MPLAB C32 1.11B與1.11A版 UART enum Label 對應
// 1.11B			// 1.11A
// #define UART2		UART3A
// #define UART3		UART2A
// #define UART4		UART1B
// #define UART5		UART3B
// #define UART6		UART2B
// UART1 與 UART1A 已在Header File中定義, (Ex: Ref. p32mx795f512l.h )
#define SPIChannel SPI_CHANNEL1	/* SPI_CHANNEL1 ~ SPI_CHANNEL4 */
// MPLAB C32 新舊版 SPI 編號對應
// New		// Old
// SPI1		SPI1
// SPI2		SPI2
// SPI3		SPI1A
// SPI4		SPI3A
#define EE_CS LATFbits.LATF3
#define EE_CS_TRIS TRISFbits.TRISF3

void UART_PutROMString(unsigned char * String);
void SPI_EEEnableWrite(void);
void SPI_DisableWriteProtect(void);
void SPI_EEByteWirte(unsigned int Addr, unsigned char Data);
unsigned char SPI_EEByteRead(unsigned int Addr);

const unsigned char LCMString1[ ] = "APP-2013TM Test.";
const unsigned char LCMString2[ ] = "Press any Key...";
const unsigned char LCMString3[ ] = "VR:    ,Rx:     ";
const unsigned char LCMString4[ ] = "A:     ,WR:  /  ";

unsigned int VRValue = 0;
unsigned char UARTString1[ 32 ];
volatile unsigned char UART_IsReceived = 0;
volatile unsigned char UART_ReceiveBuffer = 0;
unsigned char Cursor = 11, Hours = 0, Mins = 0, Secs = 0;
unsigned char TimerTickFlag = 0;
unsigned char EERWTickFlag = 0;
unsigned int SPIEEAddr = 0;

void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    static unsigned int T1IntCounter = 0;

    // 清除Timer中斷旗標。
    mT1ClearIntFlag();

    //手動觸發ADC轉換。
    ConvertADC10();

    EERWTickFlag++;

    // LED狀態反向。
    mPORTBToggleBits(BIT_9 | BIT_10 | BIT_12 | BIT_13);

    if (++T1IntCounter > 9)
    {
        T1IntCounter = 0;

        TimerTickFlag++;
    }
}

void __ISR(_UART_2_VECTOR, ipl5) ISR_UART2(void)
{
    // 使用C32 UART Function讀取接收到的資料。
    // 注意 : 此處資料僅簡單使用單一變數儲存，若資料瞬間流量大時，
    // 必須透過其他軟體機制輔助，以免資料遺失，
    // 例如:建立一個Data Queue來存放收到的資料。
    UART_ReceiveBuffer = UARTGetDataByte(UARTChannel);

    UART_IsReceived = 1;

    // 清除UART 接收中斷旗標。
    INTClearFlag(INT_U2RX);
}

int main(void)
{
    // 周邊時脈 = 系統時脈 / 8。
    OSCSetPBDIV(OSC_PB_DIV_8);

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    INTEnableSystemMultiVectoredInt();

    // 初始化I/O。
    ANSELBbits.ANSB9 = 0;
    ANSELBbits.ANSB10 = 0;
    ANSELBbits.ANSB12 = 0;
    ANSELBbits.ANSB13 = 0;
    mPORTBSetPinsDigitalOut(BIT_9 | BIT_10 | BIT_12 | BIT_13); // D2, D3, D4, D5
    mPORTBSetBits(BIT_9 | BIT_12);
    mPORTBClearBits(BIT_10 | BIT_13);

    ANSELDbits.ANSD1 = 0; // BT3
    mPORTDSetPinsDigitalIn(BIT_7 | BIT_6 | BIT_0);	// BT1, BIT2 ,BT3
    mPORTCSetPinsDigitalIn(BIT_13); // BT4

    // 初始化LCD Module。
    LCM_Init();

    // 設定遊標，輸出字串。
    LCM_SetCursor(0, 0);
    LCM_PutROMString(LCMString1);
    LCM_SetCursor(0, 1);
    LCM_PutROMString(LCMString2);

    while (PORTDbits.RD7 && PORTDbits.RD6 && PORTDbits.RD0 && PORTCbits.RC13 );

    // 啟用Timer，並指定其中斷優先權。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4);

    // 初始化Timer1。
    OpenTimer1(T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT, Timer1Tick);

    // 初始化ADC。
    OpenADC10(ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF,
              ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF,
              ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy,
              ENABLE_AN11_ANA, SKIP_SCAN_ALL);

     // 設定ADC正,負通道。
    SetChanADC10(ADC_CH0_POS_SAMPLEA_AN11 | ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEB_AN0 | ADC_CH0_NEG_SAMPLEB_NVREF);

    // 初始化UART。
    PPSUnLock;
    PPSOutput(2, RPF0, U2TX); //Assign U2TX
    PPSInput(1, U2RX, RPF1); //Assign U2RX
    PPSOutput(4, RPB6, U2RTS); //Assign U2RTS
    PPSInput(3, U2CTS, RPB8); //Assign U2CTS
    PPSLock;

    UARTConfigure(UARTChannel, UART_ENABLE_PINS_TX_RX_ONLY);
    UARTSetFifoMode(UARTChannel, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UARTChannel, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UARTChannel, PBFrequency, 9600);
    UARTEnable(UARTChannel, UART_ENABLE | UART_PERIPHERAL | UART_TX | UART_RX);

    INTSetVectorPriority(INT_UART_2_VECTOR, INT_PRIORITY_LEVEL_5);
    INTSetVectorSubPriority(INT_UART_2_VECTOR, INT_SUB_PRIORITY_LEVEL_0);
    INTEnable(INT_U2RX, INT_ENABLED);

    // 初始化SPI。
    ANSELBbits.ANSB0 = 0;
    ANSELBbits.ANSB1 = 0;
    ANSELDbits.ANSD2 = 0;

    // SPI Chip Select
    mPORTFSetPinsDigitalOut(BIT_3);
    mPORTFSetBits(BIT_3);

    PPSUnLock;
    PPSOutput(3, RPB0, SDO1); //Assign SDO1
    PPSInput(2, SDI1, RPB1); //Assign SDI1
    PPSLock;

    SpiChnConfigure(SPIChannel, SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV | SPI_OPEN_MODE8 | SPI_OPEN_SIDL);
    SpiChnSetBitRate(SPIChannel, PBFrequency, 100000);
    SpiChnEnable(SPIChannel, 1);

    LCM_SetCursor(0, 0);
    LCM_PutROMString(LCMString3);
    LCM_SetCursor(0, 1);
    LCM_PutROMString(LCMString4);

    SPI_EEEnableWrite();
    SPI_DisableWriteProtect();
    while (1)
    {
        // 轉換完成 ?
        if (INTGetFlag(INT_AD1))
        {
            // 清除ADC中斷旗標
            INTClearFlag(INT_AD1);

            // 讀取ADC轉換之數值。
            VRValue = (unsigned int) ReadADC10(0);

            // 設定遊標，輸出ADC數值。
            LCM_SetCursor(3, 0);
            LCM_PutNumber(VRValue, 4);
        }

        // UART收到資料 ?
        if (UART_IsReceived)
        {
            UART_IsReceived = 0;

            // 設定遊標，輸出字串。
            LCM_SetCursor(Cursor++, 0);
            LCM_PutASCII(UART_ReceiveBuffer);

            if (Cursor > 15) Cursor = 11;
        }

        if (TimerTickFlag)
        {
            TimerTickFlag--;

            if (++Secs > 59)
            {
                Secs = 0;
                if (++Mins > 59)
                {
                    Mins = 0;
                    if (++Hours > 23)
                        Hours = 0;
                }
            }

            sprintf((char *) UARTString1, "Total Runing Time : %2d:%2d:%2d\r",
                    Hours, Mins, Secs);

            // 使用自訂函數將字串透過UART輸出。
            UART_PutROMString(UARTString1);
        }

        if (EERWTickFlag)
        {
            unsigned int i = 0;
            EERWTickFlag--;

            SPI_EEEnableWrite();

            LCM_SetCursor(1, 1);
            LCM_PutHex((SPIEEAddr >> 16) & 0x0f);
            LCM_PutHex((SPIEEAddr >> 8) & 0xff);
            LCM_PutHex(SPIEEAddr & 0xff);

            LCM_SetCursor(1, 1);
            LCM_PutASCII(':');

            LCM_SetCursor(11, 1);
            LCM_PutHex(SPIEEAddr);
            SPI_EEByteWirte(SPIEEAddr, SPIEEAddr);

            // ! EEPROM的寫入, 約需要5mS, 期間如果進行存取, 會中斷寫入工作,
            // 此處以簡單的軟體延遲處理, 但由於不同版本的編譯器,
            // 對於軟體延遲所產生的時間間隔不一, 必須特別注意!
            // 或者改採用硬體的計時來處理.
            for (i = 0; i < 50000; i++);

            LCM_SetCursor(14, 1);
            LCM_PutHex(SPI_EEByteRead(SPIEEAddr));

            if (++SPIEEAddr > 0x1ffff)
                SPIEEAddr = 0;
        }
    }

    while (1);
}

void UART_PutROMString(unsigned char * String)
{
    while (*String != 0x00)
    {
        while (!UARTTransmitterIsReady(UARTChannel));
        UARTSendDataByte(UARTChannel, *String++);
    }
}

void SPI_EEEnableWrite(void)
{
    unsigned char i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x06);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);
}

void SPI_DisableWriteProtect(void)
{
    unsigned char i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x01);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x00);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);
}

void SPI_EEByteWirte(unsigned int Addr, unsigned char Data)
{
    unsigned char i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x02);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 16) & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 8) & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Addr & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Data);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);
}

unsigned char SPI_EEByteRead(unsigned int Addr)
{
    unsigned char Data = 0;
    unsigned char i = 0;

    // CS Active
    EE_CS = 0;

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, 0x03);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 16) & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, (Addr >> 8) & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Addr & 0xff);
    while (!SpiChnDataRdy(SPIChannel));
    SpiChnReadC(SPIChannel); //Dummy Read

    while (!SpiChnTxBuffEmpty(SPIChannel));
    SpiChnWriteC(SPIChannel, Data);
    while (!SpiChnDataRdy(SPIChannel));
    Data = SpiChnReadC(SPIChannel);

    // Wait For Last Byte Transmitted
    for (i = 0; i < 10; i++);

    // CS Inactive
    EE_CS = 1;
    for (i = 0; i < 10; i++);

    return Data;
}