#include <plib.h>
#include <p32xxxx.h>
#include ".\..\APP2013TM_LCM.h"

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 20 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC3 , PGD3 , Disable SOSC
// Disable JTAG , IO One Way OFF , PMDL One Way OFF
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_20 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_1
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx3 , FSOSCEN = OFF
#pragma config JTAGEN = OFF , IOL1WAY = OFF , PMDL1WAY = OFF
#pragma config UPLLIDIV = DIV_2 , UPLLEN = ON , FUSBIDIO = OFF

#define SystemFrequency		80000000L
#define PBFrequency		SystemFrequency / 8
#define Timer1Tick		( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		10
#define I2CChannel I2C1	/* I2C1 or I2C2 for APP1632 */ 
// MPLAB C32 新舊版 I2C 編號對應
// New		// Old
// I2C1		I2C1
// I2C2		I2C2
// I2C3		I2C1A
// I2C4		I2C2A
// I2C5		I2C3A

#define DefOutVoltValueMax (3 << 3)     // 3.0V
#define DefOutVoltValueMin (1 << 3)     // 1.0V
#define DefOutVoltValueGap (1 << 1)     // 0.25V

void PMVout_Command(unsigned int SalveAddr, unsigned int Value);
void PMOperation(unsigned int SalveAddr, unsigned char On);
unsigned char basicCrc(unsigned char remainder, unsigned char byte);

const unsigned char LCMString1[ ] = "MCP19111 Control";
const unsigned char LCMString2[ ] = "Volt:     V(ADC)";

unsigned int VRValue = 0;
unsigned long VoltValue = 0;
unsigned char TimerTickFlag = 0;
unsigned char LastBT1Status = 1, LastBT3Status = 1, LastBT4Status = 1;
unsigned long OutVoltValue = (2 << 3); // 2.0V

#define I2CChannel I2C1	/* I2C1 or I2C2 for APP1632 */

void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    static unsigned int T1IntCounter = 0;

    // 清除Timer中斷旗標。
    mT1ClearIntFlag();

    //手動觸發ADC轉換。
    ConvertADC10();

    // LED狀態反向。
    mPORTBToggleBits(BIT_9 | BIT_10 | BIT_12 | BIT_13);

    if (++T1IntCounter > 9)
    {
        T1IntCounter = 0;

        TimerTickFlag++;
    }
}

int main(void)
{
    unsigned int Temporary = 0;

    // 周邊時脈 = 系統時脈 / 8。
    OSCSetPBDIV(OSC_PB_DIV_8);

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    INTEnableSystemMultiVectoredInt();

    // 初始化I/O。
    ANSELBbits.ANSB9 = 0;
    ANSELBbits.ANSB10 = 0;
    ANSELBbits.ANSB12 = 0;
    ANSELBbits.ANSB13 = 0;
    mPORTBSetPinsDigitalOut(BIT_9 | BIT_10 | BIT_12 | BIT_13); // D2, D3, D4, D5
    mPORTBSetBits(BIT_9 | BIT_12);
    mPORTBClearBits(BIT_10 | BIT_13);

    ANSELDbits.ANSD1 = 0; // BT3
    mPORTDSetPinsDigitalIn(BIT_7 | BIT_6 | BIT_0); // BT1, BIT2 ,BT3
    mPORTCSetPinsDigitalIn(BIT_13); // BT4

    // 初始化LCD Module。
    LCM_Init();

    // 設定遊標，輸出字串。
    LCM_SetCursor(0, 0);
    LCM_PutROMString(LCMString1);
    LCM_SetCursor(0, 1);
    LCM_PutROMString(LCMString2);

    // 啟用Timer，並指定其中斷優先權。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4);

    // 初始化Timer1。
    OpenTimer1(T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT, Timer1Tick);

    // 初始化ADC。
    OpenADC10(ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF,
              ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF,
              ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy,
              ENABLE_AN26_ANA, SKIP_SCAN_ALL);

    // 設定ADC正,負通道。
    AD1CHS = 0x001a0000; //AN26,VREFL,AN0,VREFL

    // 使用C32 I2C Function初始化I2C Module。
    I2CConfigure(I2CChannel, FALSE);
    I2CSetFrequency(I2CChannel, PBFrequency, 100000);
    I2CSetSlaveAddress(I2CChannel, 0x00, 0x00, I2C_USE_7BIT_ADDRESS);
    I2CEnable(I2CChannel, TRUE);

    Temporary = (OutVoltValue << 8) >> 3;
    Temporary = (((unsigned int) Temporary & 0xff) << 8) + (((unsigned int) Temporary >> 8) & 0xff);
    PMVout_Command(0x28, Temporary);
    PMOperation(0x28, TRUE);

    while (1)
    {
        // 轉換完成 ?
        if (INTGetFlag(INT_AD1))
        {
            // 清除ADC中斷旗標
            INTClearFlag(INT_AD1);

            // 讀取ADC轉換之數值。
            VRValue = (unsigned int) ReadADC10(0);
            VoltValue = (((unsigned long) VRValue * 3300) / 1023) * 2;

            LCM_SetCursor(5, 1);
            // 設定遊標，輸出ADC數值。
            if (VoltValue > 999)
                LCM_PutNumber(VoltValue / 1000, 1);
            else
                LCM_PutASCII('0');

            LCM_PutASCII('.');
            LCM_PutNumber(VoltValue % 1000, 3);
        }

        if (!PORTDbits.RD7)
        {
            if (LastBT1Status)
            {
                static unsigned char OnOff_Falg = 0;

                LastBT1Status = 0;
                OnOff_Falg ^= 0xff;
                PMOperation(0x28, OnOff_Falg);
            }
        }
        else
            LastBT1Status = 1;

        if (!PORTDbits.RD0)
        {
            if (LastBT3Status)
            {
                unsigned int Temporary = 0;

                LastBT3Status = 0;

                OutVoltValue += DefOutVoltValueGap;

                if (OutVoltValue > DefOutVoltValueMax)
                    OutVoltValue = DefOutVoltValueMax;

                Temporary = (OutVoltValue << 8) >> 3;
                Temporary = (((unsigned int) Temporary & 0xff) << 8) + (((unsigned int) Temporary >> 8) & 0xff);

                PMVout_Command(0x28, Temporary);
            }
        }
        else
            LastBT3Status = 1;

        if (!PORTCbits.RC13)
        {
            if (LastBT4Status)
            {
                unsigned int Temporary = 0;

                LastBT4Status = 0;

                OutVoltValue -= DefOutVoltValueGap;

                if (OutVoltValue < DefOutVoltValueMin)
                    OutVoltValue = DefOutVoltValueMin;

                Temporary = (OutVoltValue << 8) >> 3;
                Temporary = (((unsigned int) Temporary & 0xff) << 8) + (((unsigned int) Temporary >> 8) & 0xff);

                PMVout_Command(0x28, Temporary);
            }
        }
        else
            LastBT4Status = 1;
    }

    while (1);
}

void PMVout_Command(unsigned int SalveAddr, unsigned int Value)
{
    unsigned char CRCValue = 0;
    CRCValue = basicCrc(CRCValue, (unsigned char) SalveAddr);
    CRCValue = basicCrc(CRCValue, 0x15);
    CRCValue = basicCrc(CRCValue, (unsigned char) ((Value >> 8) & 0x00ff));
    CRCValue = basicCrc(CRCValue, (unsigned char) (Value & 0x00ff));

    while (!I2CBusIsIdle(I2CChannel));

    I2CStart(I2CChannel);
    while (!(I2CGetStatus(I2CChannel) & I2C_START));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, SalveAddr & 0xfe);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, 0x15);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, (unsigned char) ((Value >> 8) & 0x00ff));
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, (unsigned char) (Value & 0x00ff));
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, CRCValue);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    I2CStop(I2CChannel);

    I2CEnable(I2CChannel, FALSE);
    Nop();
    Nop();

    I2CEnable(I2CChannel, TRUE);
    Nop();
    Nop();
}

void PMOperation(unsigned int SalveAddr, unsigned char Enable)
{
    unsigned char CRCValue = 0;
    CRCValue = basicCrc(CRCValue, (unsigned char) SalveAddr);
    CRCValue = basicCrc(CRCValue, 0x01);
    if (Enable)
        CRCValue = basicCrc(CRCValue, 0x80);
    else
        CRCValue = basicCrc(CRCValue, 0x00);

    while (!I2CBusIsIdle(I2CChannel));

    I2CStart(I2CChannel);
    while (!(I2CGetStatus(I2CChannel) & I2C_START));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, SalveAddr & 0xfe);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, 0x01);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    if (Enable)
        I2CSendByte(I2CChannel, 0x80);
    else
        I2CSendByte(I2CChannel, 0x00);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    while (!I2CTransmitterIsReady(I2CChannel));
    I2CSendByte(I2CChannel, CRCValue);
    while (!I2CTransmissionHasCompleted(I2CChannel));
    while (!I2CByteWasAcknowledged(I2CChannel));

    I2CStop(I2CChannel);

    I2CEnable(I2CChannel, FALSE);
    Nop();
    Nop();

    I2CEnable(I2CChannel, TRUE);
    Nop();
    Nop();

}

unsigned char basicCrc(unsigned char remainder, unsigned char byte)
{
    unsigned char i = 0;
    const char POLY = 0x7;
    const char WIDTH = 8;
    const char MSB = (1 << 7);

    remainder ^= byte;

    for (i = WIDTH; i > 0; --i)
    {
        if (remainder & MSB)
            remainder = (remainder << 1) ^ POLY;
        else
            remainder = (remainder << 1);
    }
    return remainder;
}