#ifndef HARDWARE_PROFILE_PIC32MX450F256_H
#define HARDWARE_PROFILE_PIC32MX450F256_H

    //#define USE_SELF_POWER_SENSE_IO
    #define tris_self_power     #error "No Defined"
    #define self_power          1

    //#define USE_USB_BUS_SENSE_IO
    #define tris_usb_bus_sense  #error "No Defined"
    #define USB_BUS_SENSE       1 

    #define APP2013TM

    /** CPU Clock Speed Assignment *************************************/
    //#define RUN_AT_48MHZ
    //#define RUN_AT_24MHZ
    #define RUN_AT_60MHZ

    // Various clock values
    #if defined(RUN_AT_48MHZ)
        #define GetSystemClock()            48000000UL
        #define GetPeripheralClock()        48000000UL
        #define GetInstructionClock()       (GetSystemClock())
    #elif defined(RUN_AT_24MHZ)
        #define GetSystemClock()            24000000UL
        #define GetPeripheralClock()        24000000UL
        #define GetInstructionClock()       (GetSystemClock())
    #elif defined(RUN_AT_60MHZ)    
        #define GetSystemClock()            (60000000ul)
        #define GetPeripheralClock()        (GetSystemClock()) 
        #define GetInstructionClock()       (GetSystemClock())
    #else
        #error Choose a speed
    #endif

    /** LED ************************************************************/
    #define mInitAllLEDs()  ANSELBbits.ANSB9 = 0; \
                            ANSELBbits.ANSB10 = 0; \
                            ANSELBbits.ANSB12 = 0; \
                            ANSELBbits.ANSB13 = 0; \
                            mPORTBSetPinsDigitalOut(BIT_9 | BIT_10 | BIT_12 | BIT_13);
    
    #define mLED_1              LATBbits.LATB9
    #define mLED_2              LATBbits.LATB10
    #define mLED_3              LATBbits.LATB12
    #define mLED_4              LATBbits.LATB13
    
    #define mGetLED_1()         mLED_1
    #define mGetLED_2()         mLED_2
    #define mGetLED_3()         mLED_3
    #define mGetLED_4()         mLED_4

    #define mLED_1_On()         mLED_1 = 1;
    #define mLED_2_On()         mLED_2 = 1;
    #define mLED_3_On()         mLED_3 = 1;
    #define mLED_4_On()         mLED_4 = 1;
    
    #define mLED_1_Off()        mLED_1 = 0;
    #define mLED_2_Off()        mLED_2 = 0;
    #define mLED_3_Off()        mLED_3 = 0;
    #define mLED_4_Off()        mLED_4 = 0;
    
    #define mLED_1_Toggle()     mLED_1 = !mLED_1;
    #define mLED_2_Toggle()     mLED_2 = !mLED_2;
    #define mLED_3_Toggle()     mLED_3 = !mLED_3;
    #define mLED_4_Toggle()     mLED_4 = !mLED_4;
    
    /** SWITCH *********************************************************/
    #define mInitSwitch2()      mPORTDSetPinsDigitalIn(BIT_7);
    #define mInitSwitch3()      mPORTDSetPinsDigitalIn(BIT_6);
    #define mInitAllSwitches()  mInitSwitch2();mInitSwitch3();
    #define sw2                 PORTDbits.RD7
    #define sw3                 PORTDbits.RD6


    /** POT ************************************************************/
    #define mInitPOT() { \
                OpenADC10(ADC_MODULE_ON | ADC_IDLE_CONTINUE | ADC_FORMAT_INTG16 | ADC_CLK_MANUAL | ADC_AUTO_SAMPLING_ON | ADC_SAMP_OFF, \
                ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF, \
                ADC_SAMPLE_TIME_31 | ADC_CONV_CLK_SYSTEM | ADC_CONV_CLK_32Tcy, \
                ENABLE_AN11_ANA, SKIP_SCAN_ALL); \
                SetChanADC10(ADC_CH0_POS_SAMPLEA_AN11 | ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEB_AN0 | ADC_CH0_NEG_SAMPLEB_NVREF); }
                
    /** MDD File System error checks ***********************************/
    // Will generate an error if the clock speed is too low to interface to the card
    #if (GetSystemClock() < 100000)
        #error Clock speed must exceed 100 kHz
    #endif    

    /** I/O pin definitions ********************************************/
    #define INPUT_PIN 1
    #define OUTPUT_PIN 0

    /*******************************************************************/
    /******** MDD File System selection options ************************/
    /*******************************************************************/
    #define USE_PIC32
    #define USE_32BIT

    #define USB_USE_MSD
    #define USE_SD_INTERFACE_WITH_SPI

    //SPI Configuration
    #define SPI_START_CFG_1     (PRI_PRESCAL_64_1 | SEC_PRESCAL_8_1 | MASTER_ENABLE_ON | SPI_CKE_ON | SPI_SMP_ON)
    #define SPI_START_CFG_2     (SPI_ENABLE)

    // Define the SPI frequency
    #define SPI_FREQUENCY			(20000000)

    #define SD_CS				LATDbits.LATD8
    #define SD_CS_TRIS                          TRISDbits.TRISD8
    
    #define SD_CD				PORTDbits.RD11
    #define SD_CD_TRIS                          TRISDbits.TRISD11
    
    #define SD_WE				0
    #define SD_WE_TRIS                          TRISDbits.TRISD11

    // Registers for the SPI module you want to use
    #define SPICON1				SPI1CON
    #define SPISTAT				SPI1STAT
    #define SPIBUF				SPI1BUF
    #define SPISTAT_RBF                         SPI1STATbits.SPIRBF
    #define SPICON1bits                         SPI1CONbits
    #define SPISTATbits                         SPI1STATbits
    #define SPIENABLE                           SPI1CONbits.ON
    #define SPIBRG                              SPI1BRG
    // Tris pins for SCK/SDI/SDO lines
    #define SPICLOCK                            TRISDbits.TRISD2
    #define SPIIN                               TRISBbits.TRISB1
    #define SPIOUT				TRISBbits.TRISB0

    #define putcSPI                             putcSPI1
    #define getcSPI                             getcSPI1
    #define OpenSPI(config1, config2)           OpenSPI1(config1, config2)

// Will generate an error if the clock speed is too low to interface to the card
    #if (GetSystemClock() < 100000)
        #error Clock speed must exceed 100 kHz
    #endif   
#endif  //HARDWARE_PROFILE_PIC32MX450F256_H
